@@ -1,7 +1,7 @@
 <?php
 /*
     vkeyboard main window.
-    Copyright 2003, 2004 Daniel K. Imori
+    Copyright 2003, 2004, 2006 Daniel K. Imori
 
     This file is part of vkeyboard plugin.
 
@@ -126,13 +126,14 @@ if($vkeyboard_rand == 'yes') {
          $value = 'value=\'"\'';  // " sucks
       else
          $value = 'value="' . $layout[$rand_keys[$i]] . '"';
-      echo '<input style="width: 1.5em; border-style: none" ' .
+      echo '<input style="width: ' . $vkeyboard_keysize . '; border-style: none" ' .
            'type="button" name="button' . $i . '" ' . $value .
            ' onclick="window.opener.document.forms[0].' . $passformname . '.value ' .
            '+= document.keyboard.button' . $i . '.value; document.keyboard.login.focus()"' .
-           ' onmouseover="key=setTimeout(\'add_key(\\\'' . $layout[$rand_keys[$i]] .
+           ' onmouseover="key=setTimeout(\'add_key(\\\'' . 
+           ( ($layout[$rand_keys[$i]] == "'" || $layout[$rand_keys[$i]] == '\\') ? '\\\\\\' . $layout[$rand_keys[$i]] : $layout[$rand_keys[$i]] ).
            '\\\')\', timer * 1000); startcounter()"' .
-           ' onmouseout="clearTimeout(key); stopcounter()">';
+           ' onmouseout="clearTimeout(key); stopcounter()">'. "\n";
    }
 }
 else {
@@ -141,11 +142,12 @@ else {
          $value = 'value=\'"\'';  // " sucks
       else
          $value = 'value="' . $layout[$i] . '"';
-      echo '<input style="width: 1.5em; border-style: none" ' .
+      echo '<input style="width: ' . $vkeyboard_keysize . '; border-style: none" ' .
            'type="button" name="button' . $i . '" ' . $value .
            ' onclick="window.opener.document.forms[0].' . $passformname . '.value ' .
            '+= document.keyboard.button' . $i . '.value; document.keyboard.login.focus()"' .
-           ' onmouseover="key=setTimeout(\'add_key(\\\'' . $layout[$i] .
+           ' onmouseover="key=setTimeout(\'add_key(\\\'' .
+           ( ($layout[$i] == "'" || $layout[$i] == '\\') ? '\\\\\\' . $layout[$i] : $layout[$i] ).
            '\\\')\', timer * 1000); startcounter()"' .
            ' onmouseout="clearTimeout(key); stopcounter()">';
    }
